
-- See http://www.sqlskills.com/BLOGS/BOBB/post/What-exactly-does-PERCENTILE_CONT-do-anyhow.aspx

/*
The following example uses the CUME_DIST function to compute the salary percentile for 
each employee within a given department. The value returned by the CUME_DIST function 
represents the percent of employees that have a salary less than or equal to the current 
employee in the same department. 

The PERCENT_RANK function computes the percent rank of the employee's salary within a department. 
As an analytic function, for a row r, PERCENT_RANK calculates the rank of r minus 1, 
divided by 1 less than the number of rows being evaluated (the entire query result set or a partition).
*/

USE AdventureWorks2008R2;
GO
SELECT Department, LastName, Rate, 
       CUME_DIST () OVER (PARTITION BY Department ORDER BY Rate) AS CumeDist, 
	    RANK () OVER (PARTITION BY Department ORDER BY Rate) AS Rank, 
       PERCENT_RANK() OVER (PARTITION BY Department ORDER BY Rate ) AS PctRank
FROM HumanResources.vEmployeeDepartmentHistory AS edh
    INNER JOIN HumanResources.EmployeePayHistory AS e  
    ON e.BusinessEntityID = edh.BusinessEntityID
WHERE Department IN (N'Information Services',N'Document Control') 
ORDER BY Department, Rate DESC;

/*
PERCENTILE_CONT
Calculates a percentile based on a continuous distribution of the column value. 
The result is interpolated and might not be equal to any of the specific 
values in the column.

PERCENTILE_DISC
Computes a specific percentile for sorted values in an entire rowset or within distinct partitions of a rowset. 
For example, PERCENTILE_DISC (0.5) will compute the 50th percentile (that is, the median) of an expression. 
PERCENTILE_DISC calculates the percentile based on a discrete distribution of the column values; the result is 
equal to a specific value in the column.

For a given percentile value P, 
PERCENTILE_DISC sorts the values of the expression in the ORDER BY clause and returns the value with the smallest 
CUME_DIST value (with respect to the same sort specification) that is greater than or equal to P. 

The following example uses PERCENTILE_CONT and PERCENTILE_DISC to find the median employee salary 
in each department. Note that these functions may not return the same value. This is because 
PERCENTILE_CONT interpolates the appropriate value, whether or not it exists in the data set, while 
PERCENTILE_DISC always returns an actual value from the set.
*/

USE AdventureWorks2008R2;

SELECT Name AS DepartmentName
      , ph.Rate
      ,PERCENT_RANK() OVER(PARTITION BY Name ORDER BY ph.Rate) as Percent_rank
      ,CUME_DIST() OVER(PARTITION BY Name ORDER BY ph.Rate) as Cume_Dist
      ,PERCENTILE_CONT(0.5) WITHIN GROUP (ORDER BY ph.Rate) 
                            OVER (PARTITION BY Name) AS MedianCont
      ,PERCENTILE_DISC(0.5) WITHIN GROUP (ORDER BY ph.Rate) 
                            OVER (PARTITION BY Name) AS MedianDisc
FROM HumanResources.Department AS d
INNER JOIN HumanResources.EmployeeDepartmentHistory AS dh 
    ON dh.DepartmentID = d.DepartmentID
INNER JOIN HumanResources.EmployeePayHistory AS ph
    ON ph.BusinessEntityID = dh.BusinessEntityID
WHERE dh.EndDate IS NULL;