
use AdventureWorks2008R2
go

-- raw sales table
select * from Sales.SalesOrderHeader

-- non-densified table
select SalesPersonID, 
       DATEPART(yyyy, OrderDate) AS Year, 
	   DATEPART(MONTH, OrderDate) AS Month,
       SUM(SubTotal) as SalesTotal
from Sales.SalesOrderHeader
where SalesPersonID IS NOT NULL
group by SalesPersonID, DATEPART(yyyy, OrderDate), DATEPART(MONTH, OrderDate)
order by SalesPersonID, Year, Month

-- densified table
select * from Sales.SalesByPersonByMonthAll
where SalesPersonID IS NOT NULL
order by SalesPersonID, Year, Month

-- four month moving average
select SalesPersonID, Year, Month, SalesTotal,
       AVG(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS 3 PRECEDING) AS MovingAverage
from Sales.SalesByPersonByMonthAll
WHERE SalesPersonID IS NOT NULL;

-- Performance

-- Sales by salesperson averaged over current month and preceeding three months
-- pre-SQL Server 2012 way to do this
-- two scans through table
;with CTE as
(SELECT SalesPersonID, Year, Month, SalesTotal,
        ROW_NUMBER() OVER (PARTITION BY SalesPersonID ORDER BY Year, Month) as RowNum
 from Sales.SalesByPersonByMonthAll
 WHERE SalesPersonID IS NOT NULL
 )
SELECT c1.SalesPersonID, c1.Year, c1.Month, c1.SalesTotal, AVG(c2.SalesTotal) as MovingAverage
FROM CTE c1
JOIN CTE c2
ON c1.SalesPersonID = c2.SalesPersonID 
   and c2.RowNum between c1.RowNum - 3 and c1.RowNum
GROUP BY c1.SalesPersonID, c1.Year, c1.Month, c1.SalesTotal
ORDER BY c1.SalesPersonID, c1.Year, c1.Month
GO

-- Show number of WindowSpool iterators
-- Sales by salesperson averaged over current month and preceeding three months
select SalesPersonID, Year, Month, SalesTotal,
       AVG(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS 3 PRECEDING) AS MovingAverage
from Sales.SalesByPersonByMonthAll
WHERE SalesPersonID IS NOT NULL;

-- Sales by salesperson averaged over current month and preceeding 4 months
-- Note: the cutoff for 1 window spool vs 2 window spools is rows 3
select SalesPersonID, Year, Month, SalesTotal,
       AVG(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS 4 PRECEDING) AS MovingAverage
from Sales.SalesByPersonByMonthAll
WHERE SalesPersonID IS NOT NULL;

-- Averaged over current month, preceding month, and following month
select SalesPersonID, Year, Month, SalesTotal,
       AVG(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS BETWEEN 1 PRECEDING AND 1 FOLLOWING) AS MovingAverage
from Sales.SalesByPersonByMonthAll
where SalesPersonID IS NOT NULL;
GO

SET STATISTICS IO ON
GO

DBCC FREEPROCCACHE
GO

-- Cummulative sum of sales by each salesperson, ordered by month
-- single WindowSpool operator
select SalesPersonID, Year, Month, SalesTotal,
       SUM(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS UNBOUNDED PRECEDING) AS CumulativeSum
from Sales.SalesByPersonByMonthAll
where SalesPersonID IS NOT NULL;

-- performance difference between ROW and RANGE
-- you can observe this with window_spool_ondisk_warning extended event
select SalesPersonID, Year, Month, SalesTotal,
       SUM(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month RANGE UNBOUNDED PRECEDING) AS CumulativeSum
from Sales.SalesByPersonByMonthAll
where SalesPersonID IS NOT NULL;

-- Compare each month's sales with average of preceeding three months
select SalesPersonID, Year, Month, SalesTotal,
       AVG(SalesTotal) 
	   OVER (PARTITION BY SalesPersonID ORDER BY Year, Month ROWS BETWEEN 3 PRECEDING AND 1 PRECEDING) AS MovingAverage
from Sales.SalesByPersonByMonthAll
where SalesPersonID IS NOT NULL;

